/*
 * Debug.java
 * - Created by Hiroki Arimura on 10 DEC 01
*/

import java.io.*;
import java.lang.*;
import java.util.*;
import java.math.*;
import java.text.*;

class SymbolTable{
  private HashMap nameTable;	
  private HashMap countTable;	
  private int count = 0;	

  public SymbolTable() 
  {
    nameTable  = new HashMap();
    countTable = new HashMap();
    count = 0;
  }

  public void clear() {
    nameTable.clear();
    countTable.clear();
    count = 0;
  }

  public int register(String name){

    Integer id;

    if (nameTable.containsKey(name) == true){
      id = (Integer) nameTable.get(name);
      if (id == null) {
	System.out.println("register: error id==null");
	System.exit(0);
      }

      Integer co = (Integer)countTable.get(name);
      if (co == null) {
	System.out.println("register: error co==null");
	System.exit(0);
      }

      countTable.put(name, (new Integer(co.intValue() + 1)));
    }
    else {
      id = new Integer(count);
      nameTable.put(name, id);

      Integer co = new Integer(1); // The number 1 means the first occ.
      countTable.put(name, co);

      count++;
    }

    return(id.intValue());
  }

  public boolean contains(String name)
  {
    return nameTable.containsKey(name); 
  }

  public int lookup(String name){

    int id;

    if (nameTable.containsKey(name) == true){
      id = ((Integer)nameTable.get(name)).intValue();
    }
    else {
      id = (-1); 
    }
    return id;
  }

  public void reportCount(){
    Set entries = countTable.entrySet();
    Iterator iter = entries.iterator();
    while (iter.hasNext()) {
      Map.Entry entry = (Map.Entry) iter.next();
      String key    = (String) entry.getKey();
      Integer value = (Integer) entry.getValue();
      System.out.println("\t" + key + "\t" + value);
    }
  }

  public int getSize()
  {
    return nameTable.size();
  }
}

public class Debug {

  /************************************************************
   * Global Compilation Switch
   ************************************************************/
  public static final boolean VERSION_MID = true; 

  static long totaltime = 0;	
  static boolean ENABLE_DEBUG_PRINT = true;
  
  /**********************************************************************
   * Utility routines
   **********************************************************************/

  public static void report(String str) {
    System.out.print(str);
  }

  public static void reportln(String str) {
    System.out.println(str);
  }

  public static void print(String str) {
    System.out.print(str);
  }

  public static void println(String str) {
    System.out.println(str);
  }

  public static void println(Exception e) {
    System.out.println(e);
  }

  public static void println() {
    System.out.println();
  }

  public static void debug(String str) {
    if (ENABLE_DEBUG_PRINT)
      System.out.println("@debug: " + str);
  }

  public static void errorexit(String str) {
    System.out.println("@error: " + str);
    System.exit(0);
  }

  public static void exit() {
    System.exit(0);
  }

  public static String ratio_form(double x, int ncol) {
    String fx;
    NumberFormat nf = NumberFormat.getNumberInstance();
    nf.setMaximumFractionDigits(ncol);
    fx = nf.format(x);
    return fx;
  }

  /*
   * Cputime
   */
  public static void task(String str) {
    System.out.println("");
    System.out.println("##" + str);
  }

  public static long currentTime() {
    Date date = new Date();
    long time = date.getTime();
    return time;
  }

  public static long checkin() {
    return Debug.currentTime();
  }

  public static long checkout(long startTime) {
    long currtime  = Debug.currentTime();
    long stagetime = currtime - startTime;
    Debug.println("##Time (sec): " +
		  ratio_form((double)stagetime * 0.001, 3));
    return stagetime;
  }

  //End of Debug class
}
